export class Utils {
    static addClass(el, value) {
        !this.hasClass(el, value) && el.classList.add(value);
    }
    static removeClass(el, value) {
        this.hasClass(el, value) && (el.classList.remove(value));
    }
    static hasClass(el, value) {
        return el.classList.contains(value);
    }
    static toggleClass(el, value) {
        this.hasClass(el, value) ? this.removeClass(el, value) : this.addClass(el, value);
    }
    static requestAnimFrame(callback) {
        return window.requestAnimationFrame(callback)
            || window.mozRequestAnimationFrame(callback)
            || window.webkitRequestAnimationFrame(callback)
            || window.setTimeout(callback, 16);
    }
    static animate(p) {
        const defaults = {
            timing: (t) => t,
            draw: () => { },
            duration: 300,
            element: document.body,
            loop: false,
            start: performance.now(),
            before: () => { },
            complete: () => { },
        };
        const params = Object.assign(Object.assign({}, defaults), p);
        const animation_start = () => params.element.setAttribute('data-frame', this.requestAnimFrame(animation_loop));
        const animation_stop = () => window.cancelAnimationFrame(parseInt(params.element.getAttribute('data-frame')));
        const animation_loop = (time) => {
            let timeFraction = (time - params.start) / params.duration;
            if (timeFraction > 1) {
                timeFraction = 1;
            }
            const progress = params.timing(timeFraction);
            params.draw(progress);
            if (timeFraction < 1) {
                animation_start();
            }
            else {
                animation_stop();
                if (params.loop) {
                    params.start = performance.now();
                    animation_start();
                }
                else {
                    params.complete();
                }
            }
        };
        params.before();
        animation_start();
        return animation_stop;
    }
    static easeInOutQuad(time) {
        return time < 0.5
            ? 2 * time * time
            : -1 + (4 - 2 * time) * time;
    }
    static easeOutQuad(time) {
        return time * (2 - time);
    }
    static easeOutCubic(time) {
        return (--time) * time * time + 1;
    }
    static linear(time) {
        return time;
    }
    static serialize(obj) {
        return Object.keys(obj)
            .reduce((a, k) => {
            a.push(k + '=' + encodeURIComponent(obj[k]));
            return a;
        }, [])
            .join('&');
    }
    static stringToSlug(str) {
        str = str.replace(/^\s+|\s+$/g, "");
        str = str.toLowerCase();
        const from = "àáäâèéëêìíïîòóöôùúüûñç·/_,:;";
        const to = "aaaaeeeeiiiioooouuuunc------";
        for (let i = 0, len = from.length; i < len; i++) {
            str = str.replace(new RegExp(from.charAt(i), 'g'), to.charAt(i));
        }
        str = str
            .replace(/[^a-z0-9 -]/g, "")
            .replace(/\s+/g, '-')
            .replace(/-+/g, '-');
        return str;
    }
    static urlencode(element, key, list) {
        list = list || [];
        if (typeof (element) == "object")
            for (let idx in element)
                this.urlencode(element[idx], key ? key + '[' + idx + ']' : idx, list);
        else
            list.push(key + '=' + (encodeURIComponent(element)));
        return list.join('&').split("%20").join("+");
    }
    static formatMoney(val, decPlaces, decSep, thouSep) {
        decPlaces = isNaN(Math.abs(decPlaces)) ? 2 : Math.abs(decPlaces);
        decSep = typeof decSep === "undefined" ? "." : decSep;
        thouSep = typeof thouSep === "undefined" ? "," : thouSep;
        const sign = val < 0 ? "-" : "";
        const i = Math.abs(val || 0).toFixed(decPlaces);
        const j = i.length > 3 ? i.length % 3 : 0;
        return sign + (j ? i.substr(0, j) + thouSep : "") +
            i.substr(j).replace(/(\decSep{3})(?=\decSep)/g, "$1" + thouSep) +
            (decPlaces ? decSep + Math.abs(val - parseFloat(i)).toFixed(decPlaces).slice(2) : "");
    }
    static setCookie(name, value, days = 7, path = "/") {
        const date = new Date();
        date.setTime(date.getTime() + (days * 24 * 60 * 60 * 1000));
        document.cookie = `${name}=${value}; expires=${date.toUTCString()}; path=${path};`;
    }
    static readCookie(name) {
        const cookieName = `${name}=`;
        const tokens = document.cookie.split(";");
        for (let i = 0; i < tokens.length; i++) {
            let token = tokens[i];
            while (token.charAt(0) == " ")
                token = token.substr(1, token.length);
            if (token.indexOf(cookieName) == 0)
                return token.substr(cookieName.length, token.length);
        }
        return "";
    }
    static eraseCookie(name, path = "/") {
        document.cookie = `${name}=; expires=Thu, 01 Jan 1970 00:00:01 GMT; path=${path}`;
    }
}
